@@ -3,56 +3,83 @@
 #' Performs Kalman filtering and smoothing with exact diffuse initialization
 #' using univariate approach for exponential family state space models. 
 #'
-#' Notice that in case of multivariate observations, \code{v}, \code{F}, \code{Finf}, \code{K} and \code{Kinf} are
-#' usually not the same as those calculated in usual multivariate Kalman filter. As filtering is done one observation element at the time,
-#' the elements of prediction error \eqn{v_t}{v[t]} are uncorrelated, and \code{F}, \code{Finf}, \code{K} and \code{Kinf} contain only the diagonal elemens of the corresponding covariance matrices.
+#' Notice that in case of multivariate observations, \code{v}, \code{F}, \code{Finf}, \code{K} and 
+#' \code{Kinf} are usually not the same as those calculated in usual multivariate Kalman filter. 
+#' As filtering is done one observation element at the time, the elements of prediction error 
+#' \eqn{v_t}{v[t]} are uncorrelated, and \code{F}, \code{Finf}, \code{K} and \code{Kinf} contain 
+#' only the diagonal elemens of the corresponding covariance matrices.
 #'
-#' In rare cases of a very long diffuse initialization phase with highly correlated states, cumulative rounding errors
-#' in computing \code{Finf} and \code{Pinf} can sometimes cause the diffuse phase end too early.
-#' Changing the tolerance parameter \code{tol} of the model (see \code{\link{SSModel}}) to smaller (or larger) should help.
+#' In rare cases of a diffuse initialization phase with highly correlated states, 
+#' cumulative rounding errors in computing \code{Finf} and \code{Pinf} can sometimes cause the 
+#' diffuse phase end too early. Changing the tolerance parameter \code{tol} of the model 
+#' (see \code{\link{SSModel}}) to smaller (or larger) should help.
 #' 
-#' In case of non-Gaussian models with \code{nsim=0}, the smoothed estimates relate the conditional mode of \eqn{p(\alpha|y)}, and are comparable with the results from generalized linear models. 
-#' When using importance sampling (\code{nsim>0}), results correspond to the conditional mean.
+#' In case of non-Gaussian models with \code{nsim=0}, the smoothed estimates relate 
+#' the conditional mode of \eqn{p(\alpha|y)}, and are equivalent with the results from 
+#' generalized linear models. When using importance sampling (\code{nsim>0}), 
+#' results correspond to the conditional mean.
 #' 
 #' @export
 #' @param model Object of class \code{SSModel}.
-#' @param filtering Types of filtering. Possible choices are 'state', 'signal', 'invlink', and 'none'. 
-#' For Gaussian models the default is 'state', and for non-Gaussian models 'none'.  Multiple values are allowed.
-#' @param smoothing Types of smoothing. Possible choices are 'state', 'signal', 'invlink', 
-#' 'disturbance' and 'none'. Default is 'state'. For non-Gaussian models, 
-#'  option 'disturbance' is not supported, and for Gaussian models option 'invlink' is not supported 
-#'  (as it is identical with 'signals'). Multiple values are allowed. Note that filtering for non-Gaussian models with importance sampling can be very slow with large models. Also, in approximating invlink filtering only diagonals of P_mu are returned.
-#' @param simplify If FALSE, KFS returns some generally not so interesting variables from filtering and smoothing. Default is TRUE.
-#' @param transform how to transform the model in case of non-diagonal
-#' covariance matrix \eqn{H}. Defaults to \code{'svd'}. See function \code{\link{transformSSM}} for
+#' @param filtering Types of filtering. Possible choices are 'state', 'signal', 'mean', and 'none'. 
+#' Default is 'state' for Gaussian and 'none' for non-Gaussian models.  Multiple values are allowed. Note that for Gaussian models, signal is mean.
+#' Note that filtering for non-Gaussian models with importance sampling can be very slow with large
+#'  models. Also in approximating mean filtering only diagonals of P_mu are returned.
+#' @param smoothing Types of smoothing. Possible choices are 'state', 'signal', 'mean', 
+#' 'disturbance' and 'none'. Default is 'state' and 'mean'. For non-Gaussian models, 
+#'  option 'disturbance' is not supported, and for Gaussian models option 'mean' is identical to 'signal'. 
+#' Multiple values are allowed.
+#' @param simplify If FALSE and model is completely Gaussian, KFS returns some generally not so 
+#' interesting variables from filtering and smoothing. Default is TRUE.
+#' @param transform How to transform the model in case of non-diagonal
+#' covariance matrix \eqn{H}. Defaults to \code{'ldl'}. See function \code{\link{transformSSM}} for
 #' details.
-#' @param nsim Number of independent samples.  Only used for non-Gaussian model. 
-#' Default is 0, which computes the approximating Gaussian model by \code{\link{approxSSM}} and performs the usual 
-#' Gaussian smoothing so that the smoothed state estimates equals to the conditional mode of \eqn{p(\alpha_t|y)}{p(\alpha[t]|y)}.
+#' @param nsim The number of independent samples.  Only used for non-Gaussian model. 
+#' Default is 0, which computes the approximating Gaussian model by \code{\link{approxSSM}} and 
+#' performs the usual Gaussian smoothing so that the smoothed state estimates equals to the 
+#' conditional mode of \eqn{p(\alpha_t|y)}{p(\alpha[t]|y)}.
 #' @param theta Initial values for conditional mode theta. Only used for non-Gaussian model.
-#' @param maxiter Maximum number of iterations used in linearisation. Default is 25. Only used for non-Gaussian model.
+#' @param maxiter The maximum number of iterations used in linearisation. Default is 25. 
+#' Only used for non-Gaussian model.
 #'
-#' @return For Gaussian model, a list with the following components:
+#' @return What \code{KFS} returns depends on the arguments \code{filtering}, \code{smoothed} and 
+#' \code{simplify}, and whether the model is Gaussian or not:
 #' \item{model}{Original state space model.  }
 #' \item{KFS_transform}{Type of H after possible transformation.  }
-#' \item{logLik}{Value of the log-likelihood function.  }
+#' \item{logLik}{Value of the log-likelihood function. Only computed for Gaussian models. }
+#' 
 #' \item{a}{One step predictions of states, \eqn{a_t=E(\alpha_t | y_{t-1}, \ldots , y_{1})}{a[t]=E(\alpha[t] | y[t-1], \ldots , y[1])}.  }
-#' \item{P}{Covariance matrices (or the non-diffuse parts) of predicted states, \eqn{P_t=Cov(\alpha_t | y_{t-1}, \ldots , y_{1})}{P[t]=Cov(\alpha[t] | y[t-1], \ldots , y[1])}.  }
-#' \item{Pinf}{Diffuse part of \eqn{P_t}{P[t]}. }
+#' \item{P}{Covariance matrices (of the non-diffuse parts) of predicted states, 
+#' \eqn{P_t=Cov(\alpha_t | y_{t-1}, \ldots , y_{1})}{P[t]=Cov(\alpha[t] | y[t-1], \ldots , y[1])}.  }
+#' \item{Pinf}{Diffuse part of \eqn{P_t}{P[t]}. Only returned for Gaussian models.}
+#' \item{t}{Filtered estimates of signals, \eqn{E(Z_t\alpha_t | y_{t-1}, \ldots , y_{1})}{E(Z[t]\alpha[t] | y[t-1], \ldots , y[1])}. }
+#' \item{P_theta}{Covariances \eqn{Var(Z[t]\alpha_t | y_{t-1}, \ldots , y_{1}).}{Var(Z[t]\alpha[t] | y[t-1], \ldots , y[1])}. }
+#' \item{m}{Filtered estimates of \eqn{f(\theta_t) | y_{t-1}, \ldots , y_{1})}{f(\theta[t]) | y[t-1], \ldots , y[1])}, where \eqn{f} is the inverse link function.  }
+#' \item{P_mu}{Covariances \eqn{Cov(f(\theta_t)| y_{t-1}, \ldots , y_{1})}{Cov(f(\theta[t]) | y[t-1], \ldots , y[1])}. 
+#' If \code{nsim=0}, only diagonal elements (variances) are computed, using the delta method.  }
+#' \item{alphahat}{Smoothed estimates of states, \eqn{E(\alpha_t | y_1, \ldots , y_n)}{E(\alpha[t] | y[1], \ldots , y[n])}. }
+#' \item{V}{Covariances \eqn{Var(\alpha_t | y_1, \ldots , y_n)}{Var(\alpha[t] | y[1], \ldots , y[n])}. }
+#' \item{thetahat}{Smoothed estimates of signals, \eqn{E(Z_t\alpha_t | y_1, \ldots , y_n)}{E(Z[t]\alpha[t] | y[1], \ldots , y[n])}. }
+#' \item{V_theta}{Covariances \eqn{Var(Z[t]\alpha_t | y_1, \ldots , y_n).}{Var(Z[t]\alpha[t] | y[1], \ldots , y[n])}. }
+#' \item{muhat}{Smoothed estimates of \eqn{f(\theta_t) | y_1, \ldots , y_n)}{f(\theta[t]) | y[1], \ldots , y[n])}, where \eqn{f} is the inverse link function.  }
+#' \item{V_mu}{Covariances \eqn{Cov(f(\theta_t)| y_1, \ldots , y_n)}{Cov(f(\theta[t]) | y[1], \ldots , y[n])}. 
+#' If \code{nsim=0}, only diagonal elements (variances) are computed, using the delta method.  }
+#' \item{etahat}{Smoothed disturbance terms \eqn{E(\eta_t | y_1, \ldots , y_n)}{E(\eta[t] | y[1], \ldots , y[n])}. }
+#' \item{V_eta}{Covariances \eqn{Var(\eta_t | y_1, \ldots , y_n)}{Var(\eta[t] | y[1], \ldots , y[n])}. }
+#' \item{epshat}{Smoothed disturbance terms \eqn{E(\epsilon_{t,i} | y_1, \ldots , y_n)}{E(\epsilon[t,i] | y[1], \ldots , y[n])}. 
+#' Note that due to the possible diagonalization these are on transformed scale. }
+#' \item{V_eps}{Diagonal elements of \eqn{Var(\epsilon_{t} | y_1, \ldots , y_n)}{Var(\epsilon[t] | y[1], \ldots , y[n])}. 
+#' Note that due to the diagonalization the off-diagonal elements are zero. }
+#' \item{iterations}{The number of iterations used in linearization of non-Gaussian model. }
 #' \item{v}{Prediction errors \eqn{v_{t,i} = y_{t,i} - Z_{i,t}a_{t,i}, i=1,\ldots,p}{v[t,i] = y[t,i] - Z[i,t]a[t,i], i=1,\ldots,p},
-#' where \eqn{a_{t,i}=E(\alpha_t | y_{t,i-1}, \ldots, y_{t,1}, \ldots , y_{1,1})}{a[t,i]=E(\alpha[t] | y[t,i-1], \ldots, y[t,1], \ldots , y[1,1])}.  }
-#' \item{F}{Prediction error variances \eqn{Var(v_{t,i})}{Var(v[t,i])}.  }
-#' \item{Finf}{Diffuse part of \eqn{F_t}{F[t]}.  }
-#' \item{d}{The last index of diffuse phase, i.e. the non-diffuse phase began from time \eqn{d+1}.  }
-#' \item{j}{The index of last \eqn{y_{i,t}} of diffuse phase.  }
-#' \item{alphahat}{Smoothed estimates of states, \eqn{E(\alpha_t | y_1, \ldots , y_n)}{E(\alpha[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='states'}.  }
-#' \item{V}{Covariances \eqn{Var(\alpha_t | y_1, \ldots , y_n).}{Var(\alpha[t] | y[1], \ldots , y[n]).} Only computed if \code{smoothing='states'}.  }
-#' \item{thetahat}{Smoothed estimates of signals, \eqn{E(Z_t\alpha_t | y_1, \ldots , y_n)}{E(Z[t]\alpha[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='signals'}.  }
-#' \item{V_theta}{Covariances \eqn{Var(Z[t]\alpha_t | y_1, \ldots , y_n).}{Var(Z[t]\alpha[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='signals'}.  }
-#' \item{etahat}{Smoothed disturbance terms \eqn{E(\eta_t | y_1, \ldots , y_n)}{E(\eta[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='disturbances'}.  }
-#' \item{V_eta}{Covariances \eqn{Var(\eta_t | y_1, \ldots , y_n)}{Var(\eta[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='disturbances'}.  }
-#' \item{epshat}{Smoothed disturbance terms \eqn{E(\epsilon_{t,i} | y_1, \ldots , y_n)}{E(\epsilon[t,i] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='disturbances'}. Note that due to possible diagonalization, these are on transformed scale. }
-#' \item{V_eps}{Diagonal elements of \eqn{Var(\epsilon_{t} | y_1, \ldots , y_n)}{Var(\epsilon[t] | y[1], \ldots , y[n])}. Note that due to the diagonalization, off-diagonal elements are zero. Only computed if \code{smoothing='disturbances'}.  }
+#' where \eqn{a_{t,i}=E(\alpha_t | y_{t,i-1}, \ldots, y_{t,1}, \ldots , y_{1,1})}{a[t,i]=E(\alpha[t] | y[t,i-1], \ldots, y[t,1], \ldots , y[1,1])}. 
+#' Only returned for Gaussian models.  }
+#' \item{F}{Prediction error variances \eqn{Var(v_{t,i})}{Var(v[t,i])}. 
+#' Only returned for Gaussian models.  }
+#' \item{Finf}{Diffuse part of \eqn{F_t}{F[t]}. Only returned for Gaussian models.  }
+#' \item{d}{The last index of diffuse phase, i.e. the non-diffuse phase began from time \eqn{d+1}. 
+#' Only returned for Gaussian models.  }
+#' \item{j}{The index of last \eqn{y_{i,t}} of diffuse phase. Only returned for Gaussian models.  }
 #' In addition, if argument \code{simplify=FALSE}, list contains following components:
 #' \item{K}{Covariances \eqn{Cov(\alpha_{t,i}, y_{t,i} | y_{t,i-1}, \ldots, y_{t,1}, y_{t-1}, \ldots , y_{1}), \quad i=1,\ldots,p}{Cov(\alpha[t,i], y[t,i] | y[t,i-1], \ldots, y[t,1], y[t-1], \ldots , y[1]), i=1,\ldots,p}.  }
 #' \item{Kinf}{Diffuse part of \eqn{K_t}{K[t]}.  }
@@ -60,15 +87,6 @@
 #' \item{r0, r1}{Diffuse phase decomposition of \eqn{r_t}{r[t]}.  }
 #' \item{N}{Covariances \eqn{Var(r_t)}{Var(r[t])} .  }
 #' \item{N0, N1, N2}{Diffuse phase decomposition of \eqn{N_t}{N[t]}.   }
-#' @return For non-Gaussian model, a list with the following components:
-#' \item{model}{Original state space model.  }
-#' \item{alphahat}{Smoothed estimates of states, \eqn{E(\alpha_t | y_1, \ldots , y_n)}{E(\alpha[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='state'}.  }
-#' \item{V}{Covariances \eqn{Var(\alpha_t | y_1, \ldots , y_n).}{Var(\alpha[t] | y[1], \ldots , y[n]).} Only computed if \code{smoothing='state'}.  }
-#' \item{thetahat}{Smoothed estimates of signals, \eqn{E(Z_t\alpha_t | y_1, \ldots , y_n)}{E(Z[t]\alpha[t] | y[1], \ldots , y[n])}. Only computed if \code{smoothing='signal'}.  }
-#' \item{V_theta}{Covariances \eqn{Var(Z[t]\alpha_t | y_1, \ldots , y_n).}{Var(Z[t]\alpha[t] | y[1], \ldots , y[n]).} Only computed if \code{smoothing='signal'}.  }
-#' \item{muhat}{Smoothed estimates of \eqn{f(\theta_t) | y_1, \ldots , y_n)}{f(\theta[t]) | y[1], \ldots , y[n])}, where \eqn{f} is the inverse link function. Only computed if \code{smoothing='invlink'}.  }
-#' \item{V_mu}{Covariances \eqn{Cov(f(\theta_t)| y_1, \ldots , y_n).}{Cov(f(\theta[t]) | y[1], \ldots , y[n]).} Only computed if \code{smoothing='invlink'}. If \code{nsim=0}, only diagonal elements (variances) are computed, using the delta method.  }
-#' \item{iterations}{The number of iterations used in linearization.}
 #' 
 #' @references Koopman, S.J. and Durbin J. (2000).  Fast filtering and
 #' smoothing for non-stationary time series models, Journal of American
@@ -88,26 +106,30 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
   is.SSModel(model, na.check = TRUE, return.logical = FALSE)
   
   if(missing(filtering)){
-    filtering <- ifelse(all(model$distribution == "gaussian"),"state","none")
+    if(all(model$distribution == "gaussian")){
+    filtering <- "state"
+    } else filtering <- "none"
   } else{
     filtering <- match.arg(arg = filtering, 
-                           choices = c("state", "signal", "invlink", "none"), several.ok = TRUE)
-    if ("invlink" %in% filtering && all(model$distribution == "gaussian")) {
-      warning("invlink filtering is not supported for gaussian models.")
-      filtering <- filtering[filtering != "invlink"]      
+                           choices = c("state", "signal", "mean","invlink", "none"), several.ok = TRUE)
+    filtering[filtering=="invlink"]<-"mean"
+    if ("signal" %in% filtering && all(model$distribution == "gaussian")) {
+      filtering[filtering == "signal"]  <-"mean" 
+      filtering<-unique(filtering)
     }
   }
   
   if(missing(smoothing)){
-    smoothing <- "state"
+    smoothing <- c("state","mean")
   } else{
     smoothing <- match.arg(arg = smoothing, 
-                           choices = c("state", "signal", "disturbance","invlink", "none"), 
+                           choices = c("state", "signal", "disturbance","mean","invlink", "none"), 
                            several.ok = TRUE)
-    
-    if ("invlink" %in% smoothing && all(model$distribution == "gaussian")) {
-      warning("invlink smoothing is not supported for gaussian models.")
-      smoothing <- smoothing[smoothing != "invlink"]
+    smoothing[smoothing=="invlink"]<-"mean"
+    if ("signal" %in% smoothing && all(model$distribution == "gaussian")) {
+      smoothing[smoothing == "signal"]  <-"mean" 
+      smoothing<-unique(smoothing)
+      
     }
     if ("disturbance" %in% smoothing && !all(model$distribution == "gaussian")) {
       warning("disturbance smoothing is not supported for non-gaussian models.")
@@ -201,24 +223,24 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
                               P = array(0, ("state" %in% filtering) * c(m - 1, m - 1, n - 1) + 1), 
                               theta = array(0, ("signal" %in% filtering) * c(p - 1, n - 1) + 1), 
                               P_theta = array(0, ("signal" %in% filtering) * c(p - 1, p - 1, n - 1) + 1), 
-                              mu = array(0, ("invlink" %in% filtering) * c(p - 1, n - 1) + 1), 
-                              P_mu = array(0, ("invlink" %in% filtering) * c(p - 1, p - 1, n - 1) + 1), 
+                              mu = array(0, ("mean" %in% filtering) * c(p - 1, n - 1) + 1), 
+                              P_mu = array(0, ("mean" %in% filtering) * c(p - 1, p - 1, n - 1) + 1), 
                               as.integer("state" %in% filtering), as.integer("signal" %in% filtering), 
-                              as.integer("invlink" %in% filtering))
+                              as.integer("mean" %in% filtering))
         if ("state" %in% filtering) {
           out <- c(out, list(a = ts(t(filterout$a),start=start(model$y),frequency=frequency(model$y)), 
                              P = filterout$P))                
           colnames(out$a) <- rownames(model$a1)
         }
         if ("signal" %in% filtering) {
-          out <- c(out, list(theta = ts(t(filterout$theta),start=start(model$y),frequency=frequency(model$y)), 
+          out <- c(out, list(t = ts(t(filterout$theta),start=start(model$y),frequency=frequency(model$y)), 
                              P_theta = filterout$P_theta))                
-          colnames(out$theta) <- colnames(model$y)
+          colnames(out$t) <- colnames(model$y)
         }
-        if ("invlink" %in% filtering) {
-          out <- c(out, list(mu = ts(t(filterout$mu),start=start(model$y),frequency=frequency(model$y)), 
+        if ("mean" %in% filtering) {
+          out <- c(out, list(m = ts(t(filterout$mu),start=start(model$y),frequency=frequency(model$y)), 
                              P_mu = filterout$P_mu))                
-          colnames(out$mu) <- colnames(model$y)
+          colnames(out$m) <- colnames(model$y)
         }
         
         out <- c(out, iterations = filterout$maxiter)
@@ -239,10 +261,10 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
                               V = array(0, ("state" %in% smoothing) * c(m - 1, m - 1, n - 1) + 1), 
                               thetahat = array(0, ("signal" %in% smoothing) * c(p - 1, n - 1) + 1), 
                               V_theta = array(0, ("signal" %in% smoothing) * c(p - 1, p - 1, n - 1) + 1), 
-                              muhat = array(0, ("invlink" %in% smoothing) * c(p - 1, n - 1) + 1), 
-                              V_mu = array(0, ("invlink" %in% smoothing) * c(p - 1, p - 1, n - 1) + 1), 
+                              muhat = array(0, ("mean" %in% smoothing) * c(p - 1, n - 1) + 1), 
+                              V_mu = array(0, ("mean" %in% smoothing) * c(p - 1, p - 1, n - 1) + 1), 
                               as.integer("state" %in% smoothing), as.integer("signal" %in% smoothing), 
-                              as.integer("invlink" %in% smoothing))
+                              as.integer("mean" %in% smoothing))
         if ("state" %in% smoothing) {
           out <- c(out, list(alphahat = ts(t(smoothout$alphahat),start=start(model$y),frequency=frequency(model$y)), V = smoothout$V))                
           colnames(out$alphahat) <- rownames(model$a1)
@@ -251,7 +273,7 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
           out <- c(out, list(thetahat = ts(t(smoothout$thetahat),start=start(model$y),frequency=frequency(model$y)), V_theta = smoothout$V_theta))                
           colnames(out$thetahat) <- colnames(model$y)
         }
-        if ("invlink" %in% smoothing) {
+        if ("mean" %in% smoothing) {
           out <- c(out, list(muhat = ts(t(smoothout$muhat),start=start(model$y),frequency=frequency(model$y)), V_mu = smoothout$V_mu))                
           colnames(out$muhat) <- colnames(model$y)
         }
@@ -263,6 +285,7 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
       
       if (maxiter == out$iterations) 
         warning("Maximum number of iterations reached, the linearization did not converge.")
+      out$call <- match.call(expand.dots = FALSE)
       class(out) <- "KFS"
       
       return(out)
@@ -303,7 +326,7 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
     } else KFS_transform <- "none"
     
   } else KFS_transform <- "none"
-  filtersignal<-("signal"%in%filtering) || ("invlink"%in%filtering)
+  filtersignal<-("signal"%in%filtering) || ("mean"%in%filtering)
   storage.mode(ymiss) <- "integer"
   filterout <- .Fortran(fkfilter, NAOK = TRUE, model$y, ymiss, as.integer(tv), model$Z, model$H, model$T, 
                         model$R, model$Q, model$a1, P1 = model$P1, model$P1inf, as.integer(p), 
@@ -352,11 +375,11 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
           a = ts(t(filterout$a), start = start(model$y), frequency = frequency(model$y)), 
           P = filterout$P, Pinf = filterout$Pinf))        
       }
-      if("signal"%in%filtering){
+      if("mean"%in%filtering){
         colnames(filterout$theta) <- colnames(model$y)
         out <- c(out, list( 
-          theta = ts(filterout$theta, start = start(model$y), frequency = frequency(model$y)), 
-          P_theta = filterout$P_theta))
+          m = ts(filterout$theta, start = start(model$y), frequency = frequency(model$y)), 
+          P_mu = filterout$P_theta))
       }
       out <- c(out, 
                list(v = ts(t(filterout$v), start = start(model$y), frequency = frequency(model$y)), 
@@ -379,18 +402,18 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
           theta = ts(filterout$theta, start = start(model$y), frequency = frequency(model$y)), 
           P_theta = filterout$P_theta))
       }  
-      if("invlink"%in%filtering){
+      if("mean"%in%filtering){
         mu<-out$model$y
         P_mu<-array(0,c(p,p,n))
         for(i in 1:p){
           P_mu[i, i, ] <- switch(model$distribution[i], gaussian = filterout$P_theta[i, i, ],
-                                 poisson = filterout$P_theta[i, i, ] * exp(filterout$theta[, i])^2, 
+                                 poisson = filterout$P_theta[i, i, ] * (exp(filterout$theta[, i])*model$u[,i])^2, 
                                  binomial = filterout$P_theta[i, i, ] * 
                                    (exp(filterout$theta[, i])/(1 + exp(filterout$theta[, i]))^2)^2, 
-                                 gamma = filterout$p_theta[i, i, ] * exp(filterout$theta[, i])^2, 
-                                 `negative binomial` = filterout$p_theta[i, i, ] * exp(filterout$theta[, i])^2)
+                                 gamma = filterout$P_theta[i, i, ] * exp(filterout$theta[, i])^2, 
+                                 `negative binomial` = filterout$P_theta[i, i, ] * exp(filterout$theta[, i])^2)
           mu[,i] <- switch(model$distribution[i], gaussian = filterout$theta[,i], 
-                           poisson = exp(filterout$theta[,i]), 
+                           poisson = exp(filterout$theta[,i])*model$u[,i], 
                            binomial = exp(filterout$theta[,i])/(1 + exp(filterout$theta[,i])), 
                            gamma = exp(filterout$theta[,i]), `negative binomial` = exp(filterout$theta[,i]))
           
@@ -418,11 +441,11 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
                           epshat = array(0, dim = c(p, n)), V_eps = array(0, dim = c(p, n)), 
                           etahat = array(0, dim = c(k, n)), V_eta = array(0, dim = c(k, k, n)), 
                           thetahat = array(0, dim = c(p, n)), V_theta = array(0, dim = c(p, p, n)), 
-                          as.integer(KFS_transform=="ldl" && ("signal" %in% smoothing || "invlink" %in% smoothing)), 
-{if (KFS_transform=="ldl" && ("signal" %in% smoothing || "invlink" %in% smoothing)) out$model$Z else double(1)}, 
+                          as.integer(KFS_transform=="ldl" && ("signal" %in% smoothing || "mean" %in% smoothing)), 
+{if (KFS_transform=="ldl" && ("signal" %in% smoothing || "mean" %in% smoothing)) out$model$Z else double(1)}, 
                           as.integer(dim(out$model$Z)[3]>1), as.integer(KFS_transform != "augment"), 
                           as.integer("state" %in% smoothing), as.integer("disturbance" %in% smoothing), 
-                          as.integer(("signal" %in% smoothing || "invlink" %in% smoothing)))
+                          as.integer(("signal" %in% smoothing || "mean" %in% smoothing)))
     
     
     
@@ -446,12 +469,12 @@ KFS <- function(model, filtering, smoothing, simplify = TRUE, transform = c("ldl
       colnames(out$thetahat) <- rownames(model$H[, , 1])
       out$V_theta <- smoothout$V_theta
     }
-    if ("invlink" %in% smoothing) {
+    if ("mean" %in% smoothing) {
       out$muhat <- array(NA, c(n, p))
       out$V_mu <- array(0, c(p, p, n))
       for (i in 1:p) {
         out$muhat[, i] <- switch(model$distribution[i], gaussian = smoothout$thetahat[i, ], 
-                                 poisson = exp(smoothout$thetahat[i, ]), 
+                                 poisson = exp(smoothout$thetahat[i, ])*model$u[,i], 
                                  binomial = exp(smoothout$thetahat[i, ])/(1 + exp(smoothout$thetahat[i, ])), 
                                  gamma = exp(smoothout$thetahat[i, ]), 
                                  `negative binomial` = exp(smoothout$thetahat[i, ]))
